@@ -1367,7 +1367,6 @@ adblock_activate_cb (MidoriExtension* extension,
     #endif
     KatzeArray* browsers;
     MidoriBrowser* browser;
-    guint i;
     #if !HAVE_WEBKIT_RESOURCE_REQUEST
     SoupSession* session = webkit_get_default_session ();
 
@@ -1391,8 +1390,7 @@ adblock_activate_cb (MidoriExtension* extension,
     adblock_reload_rules (extension, FALSE);
 
     browsers = katze_object_get_object (app, "browsers");
-    i = 0;
-    while ((browser = katze_array_get_nth_item (browsers, i++)))
+    KATZE_ARRAY_FOREACH_ITEM (browser, browsers)
         adblock_app_add_browser_cb (app, browser, extension);
     g_signal_connect (app, "add-browser",
         G_CALLBACK (adblock_app_add_browser_cb), extension);
