@@ -27,7 +27,7 @@ page_holder_notebook_append_view (GtkWidget* notebook)
     GtkWidget* label;
 
     view = midori_view_new (NULL);
-    browser = MIDORI_BROWSER (gtk_widget_get_toplevel (notebook));
+    browser = midori_browser_get_for_widget (notebook);
     settings = katze_object_get_object (browser, "settings");
     midori_view_set_settings (MIDORI_VIEW (view), settings);
     g_object_unref (settings);
@@ -49,7 +49,7 @@ page_holder_button_jump_to_clicked_cb (GtkWidget* button,
     if (n < 0)
         n = page_holder_notebook_append_view (notebook);
 
-    browser = MIDORI_BROWSER (gtk_widget_get_toplevel (notebook));
+    browser = midori_browser_get_for_widget (notebook);
     uri = midori_browser_get_current_uri (browser);
     view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), n);
     midori_view_set_uri (MIDORI_VIEW (view), uri);
@@ -66,7 +66,7 @@ page_holder_button_add_clicked_cb (GtkWidget* button,
 
     n = page_holder_notebook_append_view (notebook);
     view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), n);
-    browser = MIDORI_BROWSER (gtk_widget_get_toplevel (notebook));
+    browser = midori_browser_get_for_widget (notebook);
     uri = midori_browser_get_current_uri (browser);
     midori_view_set_uri (MIDORI_VIEW (view), uri);
 }
@@ -139,11 +139,9 @@ page_holder_activate_cb (MidoriExtension* extension,
 {
     KatzeArray* browsers;
     MidoriBrowser* browser;
-    guint i;
 
     browsers = katze_object_get_object (app, "browsers");
-    i = 0;
-    while ((browser = katze_array_get_nth_item (browsers, i++)))
+    KATZE_ARRAY_FOREACH_ITEM (browser, browsers)
         page_holder_app_add_browser_cb (app, browser, extension);
     g_object_unref (browsers);
     g_signal_connect (app, "add-browser",
@@ -175,7 +173,7 @@ extension_init (void)
 
     MidoriExtension* extension = g_object_new (MIDORI_TYPE_EXTENSION,
         "name", _("Pageholder"),
-        "description", "",
+        "description", _("Keep one or multiple pages open in parallel to your tabs"),
         "version", "0.1",
         "authors", "Christian Dywan <christian@twotoasts.de>",
         NULL);
