@@ -21,8 +21,6 @@
     #include <unistd.h>
 #endif
 
-static gboolean offline_mode = FALSE;
-#define HAVE_WEBKIT_RESOURCE_REQUEST WEBKIT_CHECK_VERSION (1, 1, 14)
 #define MAXLENGTH 1024 * 1024
 
 static gchar*
@@ -43,7 +41,8 @@ web_cache_get_cached_path (MidoriExtension* extension,
     checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, uri, -1);
     folder = g_strdup_printf ("%c%c", checksum[0], checksum[1]);
     sub_path = g_build_path (G_DIR_SEPARATOR_S, cache_path, folder, NULL);
-    g_mkdir (sub_path, 0700);
+    /* FIXME: Wrong place? */
+    katze_mkdir_with_parents (sub_path, 0700);
     g_free (folder);
 
     encoded = soup_uri_encode (uri, "/");
@@ -61,65 +60,6 @@ web_cache_get_cached_path (MidoriExtension* extension,
 }
 
 static gboolean
-web_cache_replace_frame_uri (MidoriExtension* extension,
-                             const gchar*     uri,
-                             WebKitWebFrame*  web_frame)
-{
-    gchar* filename;
-    gboolean handled = FALSE;
-
-    filename = web_cache_get_cached_path (extension, uri);
-
-    /* g_debug ("cache lookup: %s => %s", uri, filename); */
-
-    if (g_file_test (filename, G_FILE_TEST_EXISTS))
-    {
-        gchar* data;
-        g_file_get_contents (filename, &data, NULL, NULL);
-        webkit_web_frame_load_alternate_string (web_frame, data, NULL, uri);
-        g_free (data);
-        handled = TRUE;
-    }
-
-    g_free (filename);
-    return handled;
-}
-
-static gboolean
-web_cache_navigation_decision_cb (WebKitWebView*             web_view,
-                                  WebKitWebFrame*            web_frame,
-                                  WebKitNetworkRequest*      request,
-                                  WebKitWebNavigationAction* action,
-                                  WebKitWebPolicyDecision*   decision,
-                                  MidoriExtension*           extension)
-{
-    const gchar* uri = webkit_network_request_get_uri (request);
-    if (!(uri && g_str_has_prefix (uri, "http://")))
-        return FALSE;
-    if (offline_mode == FALSE)
-        return FALSE;
-
-    return web_cache_replace_frame_uri (extension, uri, web_frame);
-}
-
-#if WEBKIT_CHECK_VERSION (1, 1, 6)
-static gboolean
-web_cache_load_error_cb (WebKitWebView*   web_view,
-                         WebKitWebFrame*  web_frame,
-                         const gchar*     uri,
-                         GError*          error,
-                         MidoriExtension* extension)
-{
-    if (offline_mode == FALSE)
-        return FALSE;
-    if (!(uri && g_str_has_prefix (uri, "http://")))
-        return FALSE;
-
-    return web_cache_replace_frame_uri (extension, uri, web_frame);
-}
-#endif
-
-static void
 web_cache_save_headers (SoupMessage* msg,
                         gchar*       filename)
 {
@@ -131,11 +71,15 @@ web_cache_save_headers (SoupMessage* msg,
 
       soup_message_headers_iter_init (&iter, hdrs);
       dscfd = g_fopen (dsc_filename, "w");
+      g_free (dsc_filename);
+      if (!dscfd)
+          return FALSE;
+
       while (soup_message_headers_iter_next (&iter, &name, &value))
           g_fprintf (dscfd, "%s: %s\n", name, value);
       fclose (dscfd);
 
-      g_free (dsc_filename);
+      return TRUE;
 }
 
 GHashTable*
@@ -144,59 +88,60 @@ web_cache_get_headers (gchar* filename)
     GHashTable* headers;
     FILE* file;
     gchar* dsc_filename;
+    gchar line[128];
 
+    if (!filename)
+        return NULL;
+
+    /* use g_access() instead of g_file_test for better performance */
+    if (g_access (filename, F_OK) != 0)
+        return NULL;
+
+    dsc_filename = g_strdup_printf ("%s.dsc", filename);
     headers = g_hash_table_new_full (g_str_hash, g_str_equal,
                                (GDestroyNotify)g_free,
                                (GDestroyNotify)g_free);
 
-    if (!filename)
-        return headers;
-    if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-        return headers;
-
-    dsc_filename = g_strdup_printf ("%s.dsc", filename);
-    if (!g_file_test (dsc_filename, G_FILE_TEST_EXISTS))
+    if (!(file = g_fopen (dsc_filename, "r")))
     {
+        g_hash_table_destroy (headers);
         g_free (dsc_filename);
-        return headers;
+        return NULL;
     }
-    if ((file = g_fopen (dsc_filename, "r")))
+    while (fgets (line, 128, file))
     {
-        gchar line[128];
-        while (fgets (line, 128, file))
-        {
-            if (line==NULL)
-                continue;
-            g_strchomp (line);
-            gchar** data;
-            data = g_strsplit (line, ":", 2);
-            if (data[0] && data[1])
-                g_hash_table_insert (headers, g_strdup (data[0]),
-                                     g_strdup (g_strchug (data[1])));
-            g_strfreev (data);
-        }
+        gchar** data;
+
+        if (line == NULL)
+            continue;
+
+        g_strchomp (line);
+        data = g_strsplit (line, ":", 2);
+        if (data[0] && data[1])
+            g_hash_table_insert (headers, g_strdup (data[0]),
+                                 g_strdup (g_strchug (data[1])));
+        g_strfreev (data);
     }
     fclose (file);
-    /* g_hash_table_destroy (headers); */
     g_free (dsc_filename);
     return headers;
 }
 
-static gboolean
+static GFile*
 web_cache_tmp_prepare (gchar* filename)
 {
-    gchar* tmp_filename = g_strdup_printf ("%s.tmp", filename);
+    GFile *file;
 
-    /* FIXME: If load was interruped we are ending up with a partical
-       cache files forever */
-    if (g_file_test (tmp_filename, G_FILE_TEST_EXISTS))
+    gchar* tmp_filename = g_strdup_printf ("%s.tmp", filename);
+    if (g_access (tmp_filename, F_OK) == 0)
     {
         g_free (tmp_filename);
-        return FALSE;
+        return NULL;
     }
-    g_file_set_contents (tmp_filename, "", -1, NULL);
+    file = g_file_new_for_path (tmp_filename);
     g_free (tmp_filename);
-    return TRUE;
+
+    return file;
 }
 
 static void
@@ -204,117 +149,150 @@ web_cache_set_content_type (SoupMessage* msg,
                             SoupBuffer*  buffer)
 {
     #if WEBKIT_CHECK_VERSION (1, 1, 15)
-    const char *ct;
+    gchar* sniffed_type;
     SoupContentSniffer* sniffer = soup_content_sniffer_new ();
-    ct = soup_content_sniffer_sniff (sniffer, msg, buffer, NULL);
-    if (!ct)
-        ct = soup_message_headers_get_one (msg->response_headers, "Content-Type");
-    if (ct)
-        g_signal_emit_by_name (msg, "content-sniffed", ct, NULL);
+    if ((sniffed_type = soup_content_sniffer_sniff (sniffer, msg, buffer, NULL)))
+    {
+        g_signal_emit_by_name (msg, "content-sniffed", sniffed_type, NULL);
+        g_free (sniffed_type);
+    }
+    else
+    {
+        const gchar* content_type = soup_message_headers_get_one (
+            msg->response_headers, "Content-Type");
+        g_signal_emit_by_name (msg, "content-sniffed", content_type, NULL);
+    }
     #endif
 }
 
 static void
-web_cache_message_finished_cb (SoupMessage* msg,
-                               gchar*       filename)
+web_cache_message_finished_cb (SoupMessage*   msg,
+                               GOutputStream* stream)
 {
     gchar* headers;
     gchar* tmp_headers;
     gchar* tmp_data;
+    gchar* filename;
 
+    filename = g_object_get_data (G_OBJECT (stream), "filename");
     headers = g_strdup_printf ("%s.dsc", filename);
     tmp_headers = g_strdup_printf ("%s.dsc.tmp", filename);
     tmp_data = g_strdup_printf ("%s.tmp", filename);
+    g_output_stream_close (stream, NULL, NULL);
 
-    g_rename (tmp_data, filename);
-    g_rename (tmp_headers, headers);
+    if (msg->status_code == SOUP_STATUS_OK)
+    {
+        g_rename (tmp_data, filename);
+        g_rename (tmp_headers, headers);
+    }
+    else
+    {
+        g_unlink (tmp_data);
+        g_unlink (tmp_headers);
+    }
 
+    g_object_unref (stream);
     g_free (headers);
     g_free (tmp_headers);
     g_free (tmp_data);
 }
 
+static void web_cache_pause_message (SoupMessage* msg)
+{
+    SoupSession* session;
+    session = g_object_get_data (G_OBJECT (msg), "session");
+    soup_session_pause_message (session, msg);
+}
+
+static void web_cache_unpause_message (SoupMessage* msg)
+{
+    SoupSession* session;
+    session = g_object_get_data (G_OBJECT (msg), "session");
+    soup_session_unpause_message (session, msg);
+}
+
 static void
 web_cache_message_got_chunk_cb (SoupMessage* msg,
                                 SoupBuffer*  chunk,
-                                gchar*       filename)
+                                GOutputStream* stream)
 {
-    GFile *file;
-    GOutputStream *stream;
-    gchar *tmp_filename;
-
     if (!chunk->data || !chunk->length)
         return;
+    /* FIXME g_output_stream_write_async (stream, chunk->data, chunk->length,
+        G_PRIORITY_DEFAULT, NULL, NULL, (gpointer)chunk->length); */
+    g_output_stream_write (stream, chunk->data, chunk->length, NULL, NULL);
+}
 
-    tmp_filename = g_strdup_printf ("%s.tmp", filename);
-    file = g_file_new_for_path (tmp_filename);
-    if ((stream = (GOutputStream*)g_file_append_to (file, 0, NULL, NULL)))
+static void
+web_cache_message_rewrite_async_cb (GFile *file,
+                                    GAsyncResult* res,
+                                    SoupMessage*  msg)
+{
+    SoupBuffer *buffer;
+    char *data;
+    gsize length;
+    GError *error = NULL;
+
+    if (g_file_load_contents_finish (file, res, &data, &length, NULL, &error))
     {
-        g_output_stream_write (stream, chunk->data, chunk->length, NULL, NULL);
-        g_object_unref (stream);
+        buffer = soup_buffer_new (SOUP_MEMORY_TEMPORARY, data, length);
+        web_cache_set_content_type (msg, buffer);
+        soup_message_body_append_buffer (msg->response_body, buffer);
+        /* FIXME? */
+        web_cache_unpause_message (msg);
+        g_signal_emit_by_name (msg, "got-chunk", buffer, NULL);
+        soup_buffer_free (buffer);
+        g_free (data);
+        soup_message_got_body (msg);
+        soup_message_finished (msg);
     }
     g_object_unref (file);
-    g_free (tmp_filename);
+    g_object_unref (msg);
 }
 
 static void
 web_cache_message_rewrite (SoupMessage*  msg,
                            gchar*        filename)
 {
-    GHashTable* cache_headers = web_cache_get_headers (filename);
     GHashTableIter iter;
-    SoupBuffer *buffer;
     gpointer key, value;
-    char *data;
-    gsize length;
+    GFile *file;
+
+    GHashTable* cache_headers = web_cache_get_headers (filename);
+    if (!cache_headers)
+        return;
 
     soup_message_set_status (msg, SOUP_STATUS_OK);
     g_hash_table_iter_init (&iter, cache_headers);
     while (g_hash_table_iter_next (&iter, &key, &value))
         soup_message_headers_replace (msg->response_headers, key, value);
     g_signal_emit_by_name (msg, "got-headers", NULL);
+    g_hash_table_destroy (cache_headers);
 
-    msg->response_body = soup_message_body_new ();
-    g_file_get_contents (filename, &data, &length, NULL);
-    if (data && length)
-    {
-        buffer = soup_buffer_new (SOUP_MEMORY_TEMPORARY, data, length);
-        web_cache_set_content_type (msg, buffer);
-        soup_message_body_append_buffer (msg->response_body, buffer);
-        g_signal_emit_by_name (msg, "got-chunk", buffer, NULL);
-        soup_buffer_free (buffer);
-    }
-    soup_message_got_body (msg);
-    g_free (data);
-
-    #if 0
-    if (offline_mode == TRUE)
-    {
-       /* Workaroung for offline mode
-       FIXME: libsoup-CRITICAL **: queue_message: assertion `item != NULL' failed */
-       SoupSession *session = webkit_get_default_session ();
-       soup_session_requeue_message (session, msg);
-    }
-    soup_message_finished (msg);
-    #endif
+    /* FIXME? It seems libsoup already said "goodbye" by the time
+       the asynchronous function is starting to send data */
+    web_cache_pause_message (msg);
+    file = g_file_new_for_path (filename);
+    g_free (filename);
+    g_object_ref (msg);
+    g_file_load_contents_async (file, NULL,
+        (GAsyncReadyCallback)web_cache_message_rewrite_async_cb, msg);
 }
 
 static void
-web_cache_mesage_got_headers_cb (SoupMessage*     msg,
-                                 MidoriExtension* extension)
+web_cache_mesage_got_headers_cb (SoupMessage* msg,
+                                 gchar*       filename)
 {
-    SoupURI* soup_uri = soup_message_get_uri (msg);
-    gchar* uri;
-    gchar* filename;
     const gchar* nocache;
     SoupMessageHeaders *hdrs = msg->response_headers;
+    const char* cl;
 
     /* Skip files downloaded by the user */
     if (g_object_get_data (G_OBJECT (msg), "midori-web-cache-download"))
         return;
 
     /* Skip big files */
-    const char* cl = soup_message_headers_get_one (hdrs, "Content-Length");
+    cl = soup_message_headers_get_one (hdrs, "Content-Length");
     if (cl && atoi (cl) > MAXLENGTH)
         return;
 
@@ -327,98 +305,64 @@ web_cache_mesage_got_headers_cb (SoupMessage*     msg,
         return;
     }
 
-    uri = soup_uri ? soup_uri_to_string (soup_uri, FALSE) : g_strdup ("");
-    filename = web_cache_get_cached_path (extension, uri);
     if (msg->status_code == SOUP_STATUS_NOT_MODIFIED)
     {
-        /* g_debug ("loading from cache: %s -> %s", uri, filename); */
+        /* g_debug ("loading from cache: %s", filename); */
         g_signal_handlers_disconnect_by_func (msg,
-            web_cache_mesage_got_headers_cb, extension);
+            web_cache_mesage_got_headers_cb, filename);
         web_cache_message_rewrite (msg, filename);
-        g_free (filename);
     }
     else if (msg->status_code == SOUP_STATUS_OK)
     {
-        /* g_debug ("updating cache: %s -> %s", uri, filename); */
-        if (!web_cache_tmp_prepare (filename))
+        GFile* file;
+        GOutputStream* ostream;
+
+        /* g_debug ("updating cache: %s", filename); */
+        if (!(file = web_cache_tmp_prepare (filename)))
             return;
-        web_cache_save_headers (msg, filename);
-        g_signal_connect_data (msg, "got-chunk",
-            G_CALLBACK (web_cache_message_got_chunk_cb),
-            filename, (GClosureNotify)g_free, 0);
+        if (!web_cache_save_headers (msg, filename))
+            return;
+
+        #if GLIB_CHECK_VERSION (2, 20, 0)
+        ostream = (GOutputStream*)g_file_append_to (file,
+            G_FILE_CREATE_PRIVATE | G_FILE_CREATE_REPLACE_DESTINATION, NULL, NULL);
+        #else
+        g_unlink (filename);
+        ostream = (GOutputStream*)g_file_append_to (file,
+            G_FILE_CREATE_PRIVATE, NULL, NULL);
+        #endif
+        g_object_unref (file);
+
+        if (!ostream)
+            return;
+
+        g_object_set_data_full (G_OBJECT (ostream), "filename",
+                                filename, (GDestroyNotify)g_free);
+        g_signal_connect (msg, "got-chunk",
+            G_CALLBACK (web_cache_message_got_chunk_cb), ostream);
         g_signal_connect (msg, "finished",
-            G_CALLBACK (web_cache_message_finished_cb), filename);
+            G_CALLBACK (web_cache_message_finished_cb), ostream);
     }
-    g_free (uri);
 }
 
-#if HAVE_WEBKIT_RESOURCE_REQUEST
-static void
-web_cache_resource_request_starting_cb (WebKitWebView*         web_view,
-                                        WebKitWebFrame*        web_frame,
-                                        WebKitWebResource*     web_resource,
-                                        WebKitNetworkRequest*  request,
-                                        WebKitNetworkResponse* response,
-                                        MidoriExtension*       extension)
-{
-    const gchar* uri;
-    gchar* filename;
-    /* TODO: Good place to check are we offline */
-    uri = webkit_network_request_get_uri (request);
-    if (!(uri && g_str_has_prefix (uri, "http://")))
-        return;
-
-    if (offline_mode == FALSE)
-       return;
-
-    filename = web_cache_get_cached_path (extension, uri);
-    /* g_debug ("loading %s -> %s",uri, filename); */
-    if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-    {
-        g_free (filename);
-        return;
-    }
-
-    if (!(g_strcmp0 (uri, webkit_web_frame_get_uri (web_frame))
-        && g_strcmp0 (webkit_web_data_source_get_unreachable_uri (
-                      webkit_web_frame_get_data_source (web_frame)), uri)))
-    {
-        web_cache_replace_frame_uri (extension, uri, web_frame);
-        g_free (filename);
-        return;
-    }
-
-    gchar* file_uri = g_filename_to_uri (filename, NULL, NULL);
-    webkit_network_request_set_uri (request, file_uri);
-
-    g_free (file_uri);
-    g_free (filename);
-}
-#endif
-
 static void
 web_cache_session_request_queued_cb (SoupSession*     session,
                                      SoupMessage*     msg,
                                      MidoriExtension* extension)
 {
-    /*FIXME: Should we need to free soupuri? */
     SoupURI* soup_uri = soup_message_get_uri (msg);
-    gchar* uri = soup_uri ? soup_uri_to_string (soup_uri, FALSE) : g_strdup ("");
+    gchar* uri = soup_uri_to_string (soup_uri, FALSE);
 
-    /* For now we are handling only online mode here */
-    if (offline_mode == TRUE)
-        return;
-
-    if (g_str_has_prefix (uri, "http") && !g_strcmp0 (msg->method, "GET"))
+    if (uri && g_str_has_prefix (uri, "http") && !g_strcmp0 (msg->method, "GET"))
     {
         gchar* filename = web_cache_get_cached_path (extension, uri);
-        if (offline_mode == FALSE)
-        {
-            GHashTable* cache_headers;
-            gchar* etag;
-            gchar* last_modified;
+        GHashTable* cache_headers;
+        gchar* etag;
+        gchar* last_modified;
 
-            cache_headers = web_cache_get_headers (filename);
+        cache_headers = web_cache_get_headers (filename);
+        if (cache_headers)
+        {
             etag = g_hash_table_lookup (cache_headers, "ETag");
             last_modified = g_hash_table_lookup (cache_headers, "Last-Modified");
             if (etag)
@@ -427,52 +371,16 @@ web_cache_session_request_queued_cb (SoupSession*     session,
             if (last_modified)
                 soup_message_headers_replace (msg->request_headers,
                                               "If-Modified-Since", last_modified);
-            g_signal_connect (msg, "got-headers",
-                G_CALLBACK (web_cache_mesage_got_headers_cb), extension);
+            g_hash_table_destroy (cache_headers);
+        }
+        g_object_set_data (G_OBJECT (msg), "session", session);
+        g_signal_connect (msg, "got-headers",
+                G_CALLBACK (web_cache_mesage_got_headers_cb), filename);
 
-            g_free (etag);
-            g_free (last_modified);
-            g_free (filename);
-            /* FIXME: uncoment this is leading to a crash
-              g_hash_table_destroy (cache_headers); */
-            return;
-        }
-/*
-        else
-        {
-            g_debug("queued in offline mode: %s -> %s", uri, filename);
-            if (g_file_test (filename, G_FILE_TEST_EXISTS))
-            {
-                 soup_message_set_status (msg, SOUP_STATUS_NOT_MODIFIED);
-                 web_cache_message_rewrite (msg, filename);
-            }
-        }
-*/
-        g_free (filename);
     }
     g_free (uri);
 }
 
-static void
-web_cache_add_tab_cb (MidoriBrowser*   browser,
-                      MidoriView*      view,
-                      MidoriExtension* extension)
-{
-    GtkWidget* web_view = gtk_bin_get_child (GTK_BIN (view));
-    g_signal_connect (web_view, "navigation-policy-decision-requested",
-        G_CALLBACK (web_cache_navigation_decision_cb), extension);
-
-    #if WEBKIT_CHECK_VERSION (1, 1, 6)
-    g_signal_connect (web_view, "load-error",
-        G_CALLBACK (web_cache_load_error_cb), extension);
-    #endif
-
-    #if HAVE_WEBKIT_RESOURCE_REQUEST
-    g_signal_connect (web_view, "resource-request-starting",
-        G_CALLBACK (web_cache_resource_request_starting_cb), extension);
-    #endif
-}
-
 #if WEBKIT_CHECK_VERSION (1, 1, 3)
 static void
 web_cache_add_download_cb (MidoriBrowser*   browser,
@@ -491,23 +399,11 @@ static void
 web_cache_deactivate_cb (MidoriExtension* extension,
                          MidoriBrowser*   browser);
 
-static void
-web_cache_add_tab_foreach_cb (MidoriView*      view,
-                              MidoriBrowser*   browser,
-                              MidoriExtension* extension)
-{
-    web_cache_add_tab_cb (browser, view, extension);
-}
-
 static void
 web_cache_app_add_browser_cb (MidoriApp*       app,
                               MidoriBrowser*   browser,
                               MidoriExtension* extension)
 {
-    midori_browser_foreach (browser,
-          (GtkCallback)web_cache_add_tab_foreach_cb, extension);
-    g_signal_connect (browser, "add-tab",
-        G_CALLBACK (web_cache_add_tab_cb), extension);
     #if WEBKIT_CHECK_VERSION (1, 1, 3)
     g_signal_connect (browser, "add-download",
         G_CALLBACK (web_cache_add_download_cb), extension);
@@ -516,18 +412,6 @@ web_cache_app_add_browser_cb (MidoriApp*       app,
         G_CALLBACK (web_cache_deactivate_cb), browser);
 }
 
-static void
-web_cache_deactivate_tabs (MidoriView*      view,
-                           MidoriExtension* extension)
-{
-    #if HAVE_WEBKIT_RESOURCE_REQUEST
-    GtkWidget* web_view = gtk_bin_get_child (GTK_BIN (view));
-
-    g_signal_handlers_disconnect_by_func (
-       web_view, web_cache_resource_request_starting_cb, extension);
-    #endif
-}
-
 static void
 web_cache_deactivate_cb (MidoriExtension* extension,
                          MidoriBrowser*   browser)
@@ -541,13 +425,10 @@ web_cache_deactivate_cb (MidoriExtension* extension,
         extension, web_cache_deactivate_cb, browser);
     g_signal_handlers_disconnect_by_func (
         app, web_cache_app_add_browser_cb, extension);
-    g_signal_handlers_disconnect_by_func (
-       browser, web_cache_add_tab_cb, extension);
     #if WEBKIT_CHECK_VERSION (1, 1, 3)
     g_signal_handlers_disconnect_by_func (
         browser, web_cache_add_download_cb, extension);
     #endif
-    midori_browser_foreach (browser, (GtkCallback)web_cache_deactivate_tabs, extension);
 }
 
 static void
@@ -557,7 +438,6 @@ web_cache_activate_cb (MidoriExtension* extension,
     const gchar* cache_path = midori_extension_get_string (extension, "path");
     KatzeArray* browsers;
     MidoriBrowser* browser;
-    guint i;
     SoupSession* session = webkit_get_default_session ();
 
     katze_mkdir_with_parents (cache_path, 0700);
@@ -565,8 +445,7 @@ web_cache_activate_cb (MidoriExtension* extension,
                       G_CALLBACK (web_cache_session_request_queued_cb), extension);
 
     browsers = katze_object_get_object (app, "browsers");
-    i = 0;
-    while ((browser = katze_array_get_nth_item (browsers, i++)))
+    KATZE_ARRAY_FOREACH_ITEM (browser, browsers)
         web_cache_app_add_browser_cb (app, browser, extension);
     g_signal_connect (app, "add-browser",
         G_CALLBACK (web_cache_app_add_browser_cb), extension);
