@@ -12,7 +12,6 @@
 
 #include "katze-arrayaction.h"
 
-#include "katze-net.h"
 #include "katze-utils.h"
 #include "marshal.h"
 
@@ -20,12 +19,16 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
+#if HAVE_CONFIG_H
+    #include "config.h"
+#endif
+
 struct _KatzeArrayAction
 {
     GtkAction parent_instance;
 
     KatzeArray* array;
-    KatzeNet* net;
+    gboolean reversed;
 };
 
 struct _KatzeArrayActionClass
@@ -39,12 +42,14 @@ enum
 {
     PROP_0,
 
-    PROP_ARRAY
+    PROP_ARRAY,
+    PROP_REVERSED
 };
 
 enum
 {
     POPULATE_POPUP,
+    POPULATE_FOLDER,
     ACTIVATE_ITEM,
     ACTIVATE_ITEM_ALT,
     LAST_SIGNAL
@@ -100,6 +105,43 @@ katze_array_action_class_init (KatzeArrayActionClass* class)
                                        G_TYPE_NONE, 1,
                                        GTK_TYPE_MENU);
 
+    /**
+     * KatzeArrayAction::populate-folder:
+     * @array: the object on which the signal is emitted
+     * @menu: the menu shell being opened
+     * @folder: the folder being opened
+     *
+     * A context menu is going to be opened for @folder,
+     * the provided @menu can be populated accordingly.
+     *
+     * Unlike "populate-popup" this signal is emitted for
+     * the toplevel folder and all subfolders.
+     *
+     * Return value: %TRUE if the event was handled. If %FALSE is returned,
+     *               the default "populate-popup" signal is emitted.
+     *
+     * Since: 0.2.8
+     **/
+
+    signals[POPULATE_FOLDER] = g_signal_new ("populate-folder",
+                                       G_TYPE_FROM_CLASS (class),
+                                       (GSignalFlags) (G_SIGNAL_RUN_LAST),
+                                       0,
+                                       0,
+                                       NULL,
+                                       midori_cclosure_marshal_BOOLEAN__OBJECT_OBJECT,
+                                       G_TYPE_BOOLEAN, 2,
+                                       GTK_TYPE_MENU_SHELL, KATZE_TYPE_ITEM);
+
+    /**
+     * KatzeArrayAction::activate-item:
+     * @array: the object on which the signal is emitted
+     * @item: the item being activated
+     *
+     * An item was clicked with the first button.
+     *
+     * Deprecated: 0.2.8: Use "activate-item-alt" instead.
+     **/
     signals[ACTIVATE_ITEM] = g_signal_new ("activate-item",
                                        G_TYPE_FROM_CLASS (class),
                                        (GSignalFlags) (G_SIGNAL_RUN_LAST),
@@ -116,8 +158,7 @@ katze_array_action_class_init (KatzeArrayActionClass* class)
      * @item: the item being activated
      * @button: the mouse button pressed
      *
-     * An item was clicked with a particular button. Use this if you need
-     * to handle middle or right clicks specially.
+     * An item was clicked, with the specified @button.
      *
      * Return value: %TRUE if the event was handled. If %FALSE is returned,
      *               the default "activate-item" signal is emitted.
@@ -130,7 +171,7 @@ katze_array_action_class_init (KatzeArrayActionClass* class)
                                        0,
                                        0,
                                        NULL,
-                                       katze_cclosure_marshal_BOOLEAN__OBJECT_UINT,
+                                       midori_cclosure_marshal_BOOLEAN__OBJECT_UINT,
                                        G_TYPE_BOOLEAN, 2,
                                        KATZE_TYPE_ITEM, G_TYPE_UINT);
 
@@ -154,13 +195,29 @@ katze_array_action_class_init (KatzeArrayActionClass* class)
                                      "The array the action represents",
                                      KATZE_TYPE_ARRAY,
                                      G_PARAM_READWRITE));
+
+    /**
+     * KatzeArrayAction:reversed:
+     *
+     * Whether the array should be walked backwards when building menus.
+     *
+     * Since: 0.2.2
+     **/
+    g_object_class_install_property (gobject_class,
+                                     PROP_REVERSED,
+                                     g_param_spec_boolean (
+                                     "reversed",
+                                     "Reversed",
+        "Whether the array should be walked backwards when building menus",
+                                     FALSE,
+                                     G_PARAM_READWRITE));
 }
 
 static void
 katze_array_action_init (KatzeArrayAction* array_action)
 {
     array_action->array = NULL;
-    array_action->net = katze_net_new ();
+    array_action->reversed = FALSE;
 }
 
 static void
@@ -169,7 +226,6 @@ katze_array_action_finalize (GObject* object)
     KatzeArrayAction* array_action = KATZE_ARRAY_ACTION (object);
 
     katze_object_assign (array_action->array, NULL);
-    katze_object_assign (array_action->net, NULL);
 
     G_OBJECT_CLASS (katze_array_action_parent_class)->finalize (object);
 }
@@ -187,6 +243,9 @@ katze_array_action_set_property (GObject*      object,
     case PROP_ARRAY:
         katze_array_action_set_array (array_action, g_value_get_object (value));
         break;
+    case PROP_REVERSED:
+        array_action->reversed = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -206,6 +265,9 @@ katze_array_action_get_property (GObject*    object,
     case PROP_ARRAY:
         g_value_set_object (value, array_action->array);
         break;
+    case PROP_REVERSED:
+        g_value_set_boolean (value, array_action->reversed);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -232,12 +294,24 @@ katze_array_action_activate (GtkAction* action)
         GTK_ACTION_CLASS (katze_array_action_parent_class)->activate (action);
 }
 
+static void
+katze_array_action_activate_item (KatzeArrayAction* action,
+                                  KatzeItem*        item,
+                                  gint              button)
+{
+    gboolean handled = FALSE;
+    g_signal_emit (action, signals[ACTIVATE_ITEM_ALT], 0, item,
+                   button, &handled);
+    if (!handled)
+        g_signal_emit (action, signals[ACTIVATE_ITEM], 0, item);
+}
+
 static void
 katze_array_action_menu_activate_cb  (GtkWidget*        proxy,
                                       KatzeArrayAction* array_action)
 {
     KatzeItem* item = g_object_get_data (G_OBJECT (proxy), "KatzeItem");
-    g_signal_emit (array_action, signals[ACTIVATE_ITEM], 0, item);
+    katze_array_action_activate_item (array_action, item, 1);
 }
 
 static gboolean
@@ -246,13 +320,8 @@ katze_array_action_menu_button_press_cb (GtkWidget*        proxy,
                                          KatzeArrayAction* array_action)
 {
     KatzeItem* item = g_object_get_data (G_OBJECT (proxy), "KatzeItem");
-    gboolean handled;
 
-    g_signal_emit (array_action, signals[ACTIVATE_ITEM_ALT], 0, item,
-        event->button, &handled);
-
-    if (!handled)
-        g_signal_emit (array_action, signals[ACTIVATE_ITEM], 0, item);
+    katze_array_action_activate_item (array_action, item, event->button);
 
     /* we need to block the 'activate' handler which would be called
      * otherwise as well */
@@ -266,13 +335,33 @@ static void
 katze_array_action_menu_item_select_cb (GtkWidget*        proxy,
                                         KatzeArrayAction* array_action);
 
-static void
+/**
+ * katze_array_action_generate_menu:
+ * @array_action: a #KatzeArrayAction
+ * @folder: the folder to represent
+ * @menu: the menu shell to populate
+ * @proxy: the proxy, or alternatively a widget in the same window
+ *
+ * Generates menu items according to @folder, in the way they
+ * appear in automatically generated action proxies.
+ * The primary use is for implementing "populate-folder".
+ *
+ * It is worth noting that @folder can be any folder and can
+ * be generated dynamically if needed.
+ *
+ * The @proxy widget must be a related widget on the same screen,
+ * but doesn't have to be a proxy of the action.
+ *
+ * Since: 0.2.8
+ **/
+void
 katze_array_action_generate_menu (KatzeArrayAction* array_action,
                                   KatzeArray*       array,
-                                  GtkWidget*        menu,
+                                  GtkMenuShell*     menu,
                                   GtkWidget*        proxy)
 {
-    guint i;
+    gint i;
+    gint summand;
     KatzeItem* item;
     GtkWidget* menuitem;
     const gchar* icon_name;
@@ -280,15 +369,27 @@ katze_array_action_generate_menu (KatzeArrayAction* array_action,
     GtkWidget* image;
     GtkWidget* submenu;
 
-    i = 0;
-    while ((item = katze_array_get_nth_item (array, i++)))
+    if (!KATZE_IS_ARRAY (array))
+        return;
+
+    if (array_action->reversed)
+    {
+        i = katze_array_get_length (array);
+        summand = -1;
+    }
+    else
+    {
+        i = -1;
+        summand = +1;
+    }
+    while ((item = katze_array_get_nth_item (array, i += summand)))
     {
         /* FIXME: The menu item should reflect changes to the item  */
-        if (!KATZE_IS_ARRAY (item) && !katze_item_get_uri (item))
+        if (KATZE_ITEM_IS_SEPARATOR (item))
         {
             menuitem = gtk_separator_menu_item_new ();
             gtk_widget_show (menuitem);
-            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+            gtk_menu_shell_append (menu, menuitem);
             continue;
         }
         menuitem = katze_image_menu_item_new_ellipsized (
@@ -297,19 +398,22 @@ katze_array_action_generate_menu (KatzeArrayAction* array_action,
             image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
         else
         {
-            if (KATZE_IS_ARRAY (item))
+            if (KATZE_ITEM_IS_FOLDER (item))
                 icon = gtk_widget_render_icon (menuitem,
                     GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU, NULL);
             else
-                icon = katze_net_load_icon (array_action->net,
-                    katze_item_get_uri (item), NULL, proxy, NULL);
+                icon = katze_load_cached_icon (katze_item_get_uri (item), proxy);
             image = gtk_image_new_from_pixbuf (icon);
             g_object_unref (icon);
         }
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+        #if GTK_CHECK_VERSION (2, 16, 0)
+        gtk_image_menu_item_set_always_show_image (
+            GTK_IMAGE_MENU_ITEM (menuitem), TRUE);
+        #endif
+        gtk_menu_shell_append (menu, menuitem);
         g_object_set_data (G_OBJECT (menuitem), "KatzeItem", item);
-        if (KATZE_IS_ARRAY (item))
+        if (KATZE_ITEM_IS_FOLDER (item))
         {
             submenu = gtk_menu_new ();
             gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
@@ -318,19 +422,12 @@ katze_array_action_generate_menu (KatzeArrayAction* array_action,
         }
         else
         {
-            g_signal_connect (menuitem, "button-press-event",
-                G_CALLBACK (katze_array_action_menu_button_press_cb), array_action);
             /* we need the 'activate' signal as well for keyboard events */
             g_signal_connect (menuitem, "activate",
                 G_CALLBACK (katze_array_action_menu_activate_cb), array_action);
         }
-        gtk_widget_show (menuitem);
-    }
-    if (!i)
-    {
-        menuitem = gtk_image_menu_item_new_with_label (_("Empty"));
-        gtk_widget_set_sensitive (menuitem, FALSE);
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+        g_signal_connect (menuitem, "button-press-event",
+            G_CALLBACK (katze_array_action_menu_button_press_cb), array_action);
         gtk_widget_show (menuitem);
     }
 }
@@ -341,13 +438,15 @@ katze_array_action_menu_item_select_cb (GtkWidget*        proxy,
 {
     GtkWidget* menu;
     KatzeArray* array;
+    gboolean handled;
 
     menu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (proxy));
     gtk_container_foreach (GTK_CONTAINER (menu),
         (GtkCallback)(gtk_widget_destroy), NULL);
 
     array = g_object_get_data (G_OBJECT (proxy), "KatzeItem");
-    katze_array_action_generate_menu (array_action, array, menu, proxy);
+    katze_array_action_generate_menu (array_action, array, GTK_MENU_SHELL (menu), proxy);
+    g_signal_emit (array_action, signals[POPULATE_FOLDER], 0, menu, array, &handled);
 }
 
 static void
@@ -356,29 +455,50 @@ katze_array_action_proxy_clicked_cb (GtkWidget*        proxy,
 {
     GtkWidget* menu;
     KatzeArray* array;
+    gboolean handled = FALSE;
 
     if (GTK_IS_MENU_ITEM (proxy))
     {
         g_object_set_data (G_OBJECT (proxy), "KatzeItem", array_action->array);
         katze_array_action_menu_item_select_cb (proxy, array_action);
-        g_signal_emit (array_action, signals[POPULATE_POPUP], 0,
-            gtk_menu_item_get_submenu (GTK_MENU_ITEM (proxy)));
+        g_signal_emit (array_action, signals[POPULATE_FOLDER], 0,
+                       gtk_menu_item_get_submenu (GTK_MENU_ITEM (proxy)),
+                       array_action->array, &handled);
+        if (!handled)
+            g_signal_emit (array_action, signals[POPULATE_POPUP], 0,
+                gtk_menu_item_get_submenu (GTK_MENU_ITEM (proxy)));
+        return;
+    }
+
+    array = (KatzeArray*)g_object_get_data (G_OBJECT (proxy), "KatzeArray");
+    if (KATZE_IS_ITEM (array) && katze_item_get_uri ((KatzeItem*)array))
+    {
+        katze_array_action_activate_item (array_action, KATZE_ITEM (array), 1);
         return;
     }
 
     menu = gtk_menu_new ();
 
-    array = (KatzeArray*)g_object_get_data (G_OBJECT (proxy), "KatzeArray");
     if (!array)
         array = array_action->array;
-    katze_array_action_generate_menu (array_action, array, menu, proxy);
+    katze_array_action_generate_menu (array_action, array, GTK_MENU_SHELL (menu), proxy);
 
-    /* populate-popup should only affect the main proxy */
-    if (array == array_action->array)
-        g_signal_emit (array_action, signals[POPULATE_POPUP], 0, menu);
+    g_signal_emit (array_action, signals[POPULATE_FOLDER], 0, menu, array, &handled);
+    if (!handled)
+    {
+        /* populate-popup should only affect the main proxy */
+        if (array == array_action->array)
+            g_signal_emit (array_action, signals[POPULATE_POPUP], 0, menu);
+    }
 
+    #if HAVE_HILDON
+    /* Avoid a bug in GTK+ messing up the initial scrolling position */
+    katze_widget_popup (NULL, GTK_MENU (menu),
+                        NULL, KATZE_MENU_POSITION_LEFT);
+    #else
     katze_widget_popup (GTK_WIDGET (proxy), GTK_MENU (menu),
                         NULL, KATZE_MENU_POSITION_LEFT);
+    #endif
 }
 
 static GtkWidget*
@@ -395,7 +515,7 @@ katze_array_action_create_tool_item (GtkAction* action)
 {
     GtkWidget* toolitem;
 
-    toolitem = GTK_WIDGET (gtk_tool_button_new (NULL, NULL));
+    toolitem = GTK_WIDGET (gtk_tool_button_new (NULL, ""));
     return toolitem;
 }
 
@@ -454,10 +574,9 @@ katze_array_action_item_notify_cb (KatzeItem*   item,
             gtk_tool_item_set_tooltip_text (toolitem,
                 katze_item_get_uri (item));
     }
-    else if (!KATZE_IS_ARRAY (item) && !strcmp (property, "uri"))
+    else if (KATZE_ITEM_IS_BOOKMARK (item) && !strcmp (property, "uri"))
     {
-        icon = katze_net_load_icon (array_action->net, katze_item_get_uri (item),
-            NULL, GTK_WIDGET (toolitem), NULL);
+        icon = katze_load_cached_icon (katze_item_get_uri (item), GTK_WIDGET (toolitem));
         image = gtk_image_new_from_pixbuf (icon);
         g_object_unref (icon);
         gtk_widget_show (image);
@@ -489,27 +608,48 @@ katze_array_action_proxy_create_menu_proxy_cb (GtkWidget* proxy,
         image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
     else
     {
-        if (KATZE_IS_ARRAY (item))
+        if (KATZE_ITEM_IS_FOLDER (item))
             icon = gtk_widget_render_icon (menuitem,
                 GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU, NULL);
         else
-            icon = katze_net_load_icon (array_action->net,
-                katze_item_get_uri (item), NULL, proxy, NULL);
+            icon = katze_load_cached_icon (katze_item_get_uri (item), proxy);
         image = gtk_image_new_from_pixbuf (icon);
         g_object_unref (icon);
     }
     gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), image);
+    #if GTK_CHECK_VERSION (2, 16, 0)
+    gtk_image_menu_item_set_always_show_image (
+        GTK_IMAGE_MENU_ITEM (menuitem), TRUE);
+    #endif
     g_object_set_data (G_OBJECT (menuitem), "KatzeItem", item);
-    g_signal_connect (menuitem, "button-press-event",
-        G_CALLBACK (katze_array_action_menu_button_press_cb), array_action);
-    /* we need the 'activate' signal as well for keyboard events */
-    g_signal_connect (menuitem, "activate",
-        G_CALLBACK (katze_array_action_menu_activate_cb), array_action);
+    if (KATZE_ITEM_IS_FOLDER (item))
+    {
+        GtkWidget* submenu = gtk_menu_new ();
+        gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), submenu);
+        g_signal_connect (menuitem, "select",
+            G_CALLBACK (katze_array_action_menu_item_select_cb), array_action);
+    }
+    else
+    {
+        g_signal_connect (menuitem, "button-press-event",
+            G_CALLBACK (katze_array_action_menu_button_press_cb), array_action);
+        /* we need the 'activate' signal as well for keyboard events */
+        g_signal_connect (menuitem, "activate",
+            G_CALLBACK (katze_array_action_menu_activate_cb), array_action);
+    }
     gtk_tool_item_set_proxy_menu_item (GTK_TOOL_ITEM (proxy),
         "katze-tool-item-menu", menuitem);
     return TRUE;
 }
 
+static void
+katze_array_action_toolitem_destroy_cb (GtkToolItem* toolitem,
+                                        KatzeItem*   item)
+{
+    g_signal_handlers_disconnect_by_func (item,
+        G_CALLBACK (katze_array_action_item_notify_cb), toolitem);
+}
+
 /**
  * katze_array_action_create_tool_item_for:
  * @array_action: a #KatzeArrayAction
@@ -541,18 +681,17 @@ katze_array_action_create_tool_item_for (KatzeArrayAction* array_action,
     uri = katze_item_get_uri (item);
     desc = katze_item_get_text (item);
 
-    if (!KATZE_IS_ARRAY (item) && !uri)
+    if (KATZE_ITEM_IS_SEPARATOR (item))
         return gtk_separator_tool_item_new ();
 
-    toolitem = gtk_tool_button_new (NULL, NULL);
+    toolitem = gtk_tool_button_new (NULL, "");
     g_signal_connect (toolitem, "create-menu-proxy",
         G_CALLBACK (katze_array_action_proxy_create_menu_proxy_cb), item);
-    if (KATZE_IS_ARRAY (item))
+    if (KATZE_ITEM_IS_FOLDER (item))
         icon = gtk_widget_render_icon (GTK_WIDGET (toolitem),
             GTK_STOCK_DIRECTORY, GTK_ICON_SIZE_MENU, NULL);
     else
-        icon = katze_net_load_icon (array_action->net, uri,
-            NULL, GTK_WIDGET (toolitem), NULL);
+        icon = katze_load_cached_icon (uri, GTK_WIDGET (toolitem));
     image = gtk_image_new_from_pixbuf (icon);
     g_object_unref (icon);
     gtk_widget_show (image);
@@ -577,16 +716,16 @@ katze_array_action_create_tool_item_for (KatzeArrayAction* array_action,
         gtk_tool_item_set_tooltip_text (toolitem, desc);
     else
         gtk_tool_item_set_tooltip_text (toolitem, uri);
-    if (KATZE_IS_ARRAY (item))
-    {
-        g_object_set_data (G_OBJECT (toolitem), "KatzeArray", item);
-        g_signal_connect (toolitem, "clicked",
-            G_CALLBACK (katze_array_action_proxy_clicked_cb), array_action);
-    }
+
+    g_object_set_data (G_OBJECT (toolitem), "KatzeArray", item);
+    g_signal_connect (toolitem, "clicked",
+        G_CALLBACK (katze_array_action_proxy_clicked_cb), array_action);
 
     g_object_set_data (G_OBJECT (toolitem), "KatzeArrayAction", array_action);
     g_signal_connect (item, "notify",
         G_CALLBACK (katze_array_action_item_notify_cb), toolitem);
+    g_signal_connect (toolitem, "destroy",
+        G_CALLBACK (katze_array_action_toolitem_destroy_cb), item);
     return toolitem;
 }
 
