@@ -116,6 +116,9 @@ midori_browser_set_current_uri        (MidoriBrowser*     browser,
 const gchar*
 midori_browser_get_current_uri        (MidoriBrowser*     browser);
 
+void
+midori_browser_set_current_page_smartly (MidoriBrowser* browser,
+                                         gint           n);
 void
 midori_browser_set_current_page       (MidoriBrowser*     browser,
                                        gint               n);
