@@ -136,21 +136,21 @@ midori_history_format_date (KatzeItem *item)
 
     /* A negative age is a date in the future, the clock is probably off */
     if (age < -1)
-        ;
+        sdate = g_strdup ("");
     else if (age > 7 || age < 0)
     {
         strftime (token, sizeof (token), "%x", localtime ((time_t*)&day));
         sdate = g_strdup (token);
     }
     else if (age > 6)
-        sdate = _("A week ago");
+        sdate = g_strdup (_("A week ago"));
     else if (age > 1)
         sdate = g_strdup_printf (ngettext ("%d day ago",
             "%d days ago", (gint)age), (gint)age);
     else if (age == 0)
-        sdate = _("Today");
+        sdate = g_strdup (_("Today"));
     else
-        sdate = _("Yesterday");
+        sdate = g_strdup (_("Yesterday"));
     return sdate;
 }
 
@@ -563,8 +563,11 @@ midori_history_treeview_render_text_cb (GtkTreeViewColumn* column,
         g_object_set (renderer, "markup", NULL,
                       "text", katze_item_get_name (item), NULL);
     else if (KATZE_ITEM_IS_FOLDER (item))
-        g_object_set (renderer, "markup", NULL,
-                      "text", midori_history_format_date (item), NULL);
+    {
+        gchar* formatted = midori_history_format_date (item);
+        g_object_set (renderer, "markup", NULL, "text", formatted, NULL);
+        g_free (formatted);
+    }
     else
         g_object_set (renderer, "markup", _("<i>Separator</i>"), NULL);
 
@@ -665,7 +668,6 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
         gchar* sqlcmd;
         KatzeItem* child;
         KatzeArray* array;
-        guint i = 0;
 
         db = g_object_get_data (G_OBJECT (history->array), "db");
         sqlcmd = g_strdup_printf ("SELECT uri, title, date, day "
@@ -674,7 +676,7 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
                  (int)katze_item_get_added (item));
         array = katze_array_from_sqlite (db, sqlcmd);
         g_free (sqlcmd);
-        while ((child = katze_array_get_nth_item (KATZE_ARRAY (array), i++)))
+        KATZE_ARRAY_FOREACH_ITEM (child, KATZE_ARRAY (array))
         {
             if ((uri = katze_item_get_uri (child)) && *uri)
             {
